

from .config import WARNING_UNKNOWN


from typing import  Callable, Any
import datetime
import inspect
import re


def get_module_name(func: Callable[..., Any]) -> str:
    """
    Given a function, returns the name of the module in which it is defined.
    """
    module = inspect.getmodule(func)
    if module is None:
        return ''
    else:
        return module.__name__.split('.')[-1]

def log_issue(exception: Exception, func: Callable[..., Any], additional_info: str = "") -> None:
    """
    Logs an issue. Can be called anywhere and will display an error message showing the module, the function, the exception and if specified, the additional info.

    Args:
        exception (Exception): The exception that was raised.
        func (Callable[..., Any]): The function in which the exception occurred.
        additional_info (str): Any additional information to log. Default is an empty string.

    Returns:
        None
    """
    now = datetime.datetime.now().strftime("%d/%m/%y %H:%M:%S")
    if hasattr(func, '__name__'):
        function_name = func.__name__
        module_name = get_module_name(func)
    else:
        function_name = func if isinstance(func, str) else WARNING_UNKNOWN
        print(f"🟡 What is this function? {func} * {type(func)}")
        try:
            module_name = get_module_name(func)
        except:
            module_name = "Couldn't get the module name"
    additional = f"""
    ****************************************
    Additional Info: 
    {additional_info}
    ****************************************""" if additional_info else ""
    print(f"""
    ----------------------------------------------------------------
    🚨 ERROR 🚨
    Occurred: {now}
    Module: {module_name} | Function: {function_name}
    Exception: {exception}{additional}
    ----------------------------------------------------------------
    """)

def remove_excess(text: str) -> str:
    """
    Replaces all occurrences of double newlines ('\n\n') and double spaces with single newline and space, respectively.
    """
    double_jump = '\n\n'
    double_space = '  '
    while double_jump in text:
        text = text.replace(double_jump, '\n')
    while double_space in text:
        text = text.replace(double_space, ' ')
    return text

def remove_non_printable(text :str) -> str:
    """
    Strong cleaner which removes non-ASCII characters from the input text.
    """
    text = re.sub(r'[\x00-\x1f\x7f-\x9f]', '', text) # removes non printable char
    y = text.split()
    z = [el for el in y if all(ord(e) < 128 for e in el)]
    return ' '.join(z)

def get_now(exact: bool = False) -> str:
    """
    Small function to get the timestamp in string format.
    By default we return the following format: "10_Jan_2023" but if exact is True, we will return 10_Jan_2023_@15h23s33
    """
    now = datetime.datetime.now()
    return datetime.datetime.strftime(now, "%d_%b_%Y@%Hh%Ms%S") if exact else datetime.datetime.strftime(now, "%d_%b_%Y")


# *************************************************************
if __name__ == "__main__":
    pass


# To generate prompts

from .config import DEFAULT_FILTERING_CRITERIA
from .utils import remove_excess


def gen_prompt_filter(criteria:str=DEFAULT_FILTERING_CRITERIA) -> str:
    """
    Generates the system prompt / role to filter content based on a list of criteria.
    """
    return remove_excess(f"""
        You are a rigorous analyst. 
        The user will provide you with content extracted from a website. You must analyze this content and return True if the majority of the Criteria are met.
        ONLY use the content provided and follow strictly the below Instructions:

        ### Instructions: 
        1. Take your time and think step by step.
        2. Carefully go through the below Criteria: {criteria}
        3. If the majority of the Criteria are respected, return the word True and nothing else.
        4. If the Criteria are NOT met at ALL, return the word False followed with a justification where you explain WHY the Criteria are not met at all. Be specific and say which criteria is not met and why.

        ### Important considerations
        - Follow strictly the Instructions. 
        - Return True if the majority of the Criteria are respected. In this case, do not provide any justification. Just return True.
        - If a large majority of the Criteria are STRICTLY NOT met, return the word False followed with the explanation. In this case, you MUST explain which Criteria is not met and WHY.
        - Do not preface your answer by anything.
        """)

def gen_prompt_result(json_output:str, json_example:str=None, additional_consideration:str=None) -> str:
    """
    Provide a list of criteria
    """
    additional_consideration = "" if not additional_consideration else "Very important: You MUST the below considerations in mind when performing the task:\n" + additional_consideration
    example = "" if not json_example else f"""\n### Example of output:\n{json_example}\n"""
    return remove_excess(f"""
        You are a rigorous analyst. 
        The user will provide you with content extracted from a website. You must analyze this content and return a JSON object with the information filled when available.
        ONLY use the content available, NEVER INVENT anything and follow strictly the below Instructions:

        ### Instructions: 
        1. Take your time and think step by step.
        2. Review and keep in mind ALL the fields of the below json structure: {json_output}
        3. Go through the content provided by the user and try your best to fill the json accordingly. 
        4. If the information is NOT in the content AT ALL, put NA.
        {example}
        ### Important considerations:
        - Follow strictly the Instructions. 
        - Remember that you must construct a JSON object with the provided keys, populated with the relevant information. If an information is not available at ALL or does not apply, use 'NA' for those fields.
        - Return the JSON and the JSON only. Do not preface by anything not even ```json before. 
        - Do your best, my job depends on the quality of your output.
        {additional_consideration}
        """)

def gen_role_summarizer() -> str:
    """
    Quick prompt for summarization. 
    """
    return remove_excess("""
    You are an expert at making summarizations that preserve ALL useful information.
    You take the text submitted by the user and return a detailed summary.
    You MUST ensure that you keep ALL relevant information about the main topic of the content.
                        
    ### Important Considerations: 
    - Do not include anything that is not in the provided text. ALWAYS be truthfull.
    - Avoid puting recommendations like 'Subscribe to their newsletter for more information and updates'. REMOVE ALL reference to user action like 'individuals can subscribe to their monthly newsletter'.
    - Do not preface the summary with anything. Do not put a title. Only return the summary and nothing else.
    - Ensure you keep ALL contact details (email or phone number). Do not consider a website URL as a contact information.
    - Ensure you keep ALL numbers and factual informations.           
    """)

# *************************************************************
if __name__ == "__main__":
    pass


# Functions to call openAI LLM

from .config import (OAI_KEY, MODEL_EMB_SMALL, MODEL_CHAT, MAX_TOKEN_OUTPUT_DEFAULT, 
                     MODEL_GPT4_TURBO, MODEL_GPT4_STABLE, MAX_TOKEN_WINDOW_GPT4_TURBO, 
                     MAX_TOKEN_WINDOW_GPT4, MAX_TOKEN_WINDOW_GPT35_TURBO, MAX_TOKEN_OUTPUT_GPT3, MAX_TOKEN_OUTPUT_DEFAULT_HUGE)
from .utils import log_issue


from typing import Optional
import tiktoken
import openai


# *************************************************************

YOUR_CLIENT = openai.OpenAI(
    api_key=OAI_KEY,
)

# *************************************************************

def embed_text(
    text: str, max_attempts: int = 3, model=MODEL_EMB_SMALL
) -> Optional[list[float]]:
    """
    Micro function which returns the embedding of one chunk of text or None if issue.

    Model is the small one by default.
    """
    if text == "":
        return "res"  # FIXME
    attempts = 0
    while attempts < max_attempts:
        try:
            res = (
                YOUR_CLIENT.embeddings.create(
                    model=model, input=text, encoding_format="float"
                    ).data[0].embedding
            )
            return res
        except Exception as e:
            attempts += 1
            log_issue(f"OAI Embedding faced the exception {e} at attempt # {attempts} out of 3"
            )

def ask_question_gpt(question: str, role: str = "", model: str = MODEL_CHAT, 
                     max_tokens: int = MAX_TOKEN_OUTPUT_DEFAULT, verbose: bool = True, temperature=0, top_p=1,) -> Optional[str]:
    """
    Queries an OpenAI GPT model (GPT-3.5 Turbo or GPT-4) with a specific question.

    Args:
        question (str): The question to ask the model.
        role (str, optional): System prompt to be initialized in the chat table, defining ChatGPT's behavior.
        model (str, optional): The model to use. Defaults to GPT-3.5 Turbo. To choose GPT 4, use 'MODEL_GPT4_TURBO'
        max_tokens (int, optional): Maximum number of tokens for the answer.
        verbose (bool, optional): Will print information in the console.

    Returns:
        str: The model's reply to the question or None if issue
    """
    max_token_window = {
        MODEL_GPT4_TURBO: MAX_TOKEN_WINDOW_GPT4_TURBO,
        MODEL_GPT4_STABLE: MAX_TOKEN_WINDOW_GPT4,
        MODEL_CHAT: MAX_TOKEN_WINDOW_GPT35_TURBO,
    }.get(model, MAX_TOKEN_OUTPUT_GPT3)
    initial_token_usage = calculate_token(role) + calculate_token(question)
    if initial_token_usage > max_token_window:
        print("Your input is too large for the query regardless of the max_tokens for the reply.")
        return ""
    elif initial_token_usage + max_tokens > max_token_window:
        max_tokens_adjusted = max_token_window - initial_token_usage
        print(f"Your input + the requested tokens for the answer exceed the maximum amount of {max_token_window}.\n Please adjust the max_tokens to a MAXIMUM of {max_tokens_adjusted}")
        return ""
    current_chat = initialize_role_in_chatTable(role)
    current_chat = add_content_to_chatTable(question, "user", current_chat)
    if verbose:
        print(f"Completion ~ {max_tokens} tokens. Request ~ {initial_token_usage} tokens.\nContext provided to GPT is:\n{current_chat}")
    return request_chatgpt(current_chat, max_tokens=max_tokens, model=model, temperature=temperature, top_p=top_p)


def request_chatgpt(current_chat: list, max_tokens: int, stop_list=False, max_attempts=3, model=MODEL_CHAT, temperature=0, top_p=1) -> Optional[str]:
    """
    Calls the ChatGPT OpenAI completion endpoint with specified parameters.

    Args:
        current_chat (list): The prompt used for the request.
        max_tokens (int, optional): Maximum number of tokens for the answer.
        stop_list (bool, optional): Whether to use specific stop tokens. Defaults to False.
        max_attempts (int, optional): Maximum number of retries. Defaults to 3.
        model (str, optional): ChatGPT OpenAI model used for the request. Defaults to 'MODEL_CHAT'.
        temperature (float, optional): Sampling temperature for the response. A value of 0 means deterministic output. Defaults to 0.
        top_p (float, optional): Nucleus sampling parameter, with 1 being 'take the best'. Defaults to 1.

    Returns:
        str: The response text or None if issue
    """
    max_tokens = int(max_tokens)
    if max_tokens > MAX_TOKEN_OUTPUT_GPT3:
        print(f"The Max Tokens cannot exceed {MAX_TOKEN_OUTPUT_GPT3}. You put {max_tokens}. This is the max length of the response.")
        return
    stop = stop_list if (stop_list and len(stop_list) < 4) else ""
    attempts = 0
    valid = False
    rep = None    # print("Writing the reply for ", current_chat) # Remove in production - to see what is actually fed as a prompt
    while attempts < max_attempts and not valid:
        try:
            response = YOUR_CLIENT.chat.completions.create(
                messages=current_chat,
                temperature=temperature,
                max_tokens=max_tokens,
                top_p=top_p,
                frequency_penalty=0,
                presence_penalty=0,
                stop=stop,
                model=model,
            )
            rep = response.choices[0].message.content
            rep = rep.strip()
            valid = True
        except Exception as e:
            attempts += 1
    if not rep:
        print(f"We didn't get a reply despite {attempts} attempts")
    return rep


def ask_question_gpt4(question: str, role: str, model=MODEL_GPT4_TURBO, max_tokens=MAX_TOKEN_OUTPUT_DEFAULT_HUGE, verbose=False, temperature=0, top_p=1, json_on=False,) -> str:
    """
    Queries Chat GPT 4 with a specific question if too lazy to change the param in ask_question_gpt)
    """
    return ask_question_gpt(question=question, role=role, model=model, max_tokens=max_tokens, verbose=verbose, temperature=temperature, top_p=top_p)


def calculate_token(text: str) -> Optional[int]:
    """
    Calculates the number of tokens for a given text using a specific tokenizer.

    Args:
        text (str): The text to calculate tokens for.

    Returns:
        int: The number of tokens in the text or -1 if there's an error.
    
    Note:
        Uses the tokenizer API and takes approximately 0.13 seconds per query.
    """
    if not isinstance(text, str): 
        print(f"Input is {type(text)} - must be str. Try force conversation")
        try:
            text = str(text)
        except Exception as e:
            log_issue(e, calculate_token, f"Failed to convert to string => {text}")
            return
    try:
        encoding = tiktoken.encoding_for_model("gpt-3.5-turbo")
        return len(encoding.encode(text))
    except Exception as e:
        log_issue(e, calculate_token, f"Input type: {type(text)}. Text: {text}")
        return -1

def initialize_role_in_chatTable(role_definition: str) -> list[dict[str, str]]:
    """
    We need to define how we want our model to perform.
    This function takes this definition as a input and returns it into the chat_table_format.
    """
    return [{"role": "system", "content": role_definition}]


def add_content_to_chatTable(
    content: str, role: str, chatTable: list[dict[str, str]]
) -> list[dict[str, str]]:
    """
    Feeds a chatTable with the new query. Returns the new chatTable.
    Role is either 'assistant' when the AI is answering or 'user' when the user has a question.
    Added a security in case change of name.
    """
    new_chatTable = list(chatTable)
    normalized_role = role.lower()
    if normalized_role in ["user", "client"]:
        new_chatTable.append({"role": "user", "content": content})
    else:
        new_chatTable.append({"role": "assistant", "content": content})
    return new_chatTable

# *************************************************************
if __name__ == "__main__":
    pass


#   Functions related to fetching content from the web


from .config import HEADERS, HTTP_URL_PATTERN
from .utils import log_issue, remove_excess, remove_non_printable


from urllib.parse import urlparse, urlunparse, quote, unquote
from requests.adapters import HTTPAdapter
from requests.exceptions import SSLError
from urllib3.util.retry import Retry
from urllib.parse import urljoin
from bs4 import BeautifulSoup
from collections import deque
from typing import Optional
import concurrent.futures
import requests
import random
import time
import re

# ****** SET UP SESSION OBJECT FOR SCRAPPING *******

def create_session(max_retries: int = 3, backoff_factor: float = 0.3, status_forcelist: tuple = (500, 502, 504)) -> requests.Session:
    """
    Create and configure a requests.Session object.
    """
    session = requests.Session()
    retry = Retry(
        total=max_retries,
        backoff_factor=backoff_factor,  # Set the backoff factor for exponential delay
        status_forcelist=status_forcelist, # Set the list of HTTP status codes to consider for retries
    )
    adapter = HTTPAdapter(max_retries=retry)
    session.mount("http://", adapter)
    session.mount("https://", adapter)
    session.headers.update(HEADERS)
    return session

session = create_session()

# ****************** FUNCS ******************

# Cleans / Removes about 60% of the content
def clean_soup(soup: BeautifulSoup, url: Optional[str] = None) -> str:
    """
    Clean and extract text from a BeautifulSoup object.

    Args:
        soup (BeautifulSoup): The parsed HTML or XML document to be cleaned.
        url (Optional): Default is None. Allows to have special rules for special websites.

    Returns:
        str: The cleaned text content extracted from the soup.
    """
    if ("You need to enable JavaScript to run this app." in soup.get_text()):
        # Here, we would need to use selenium and do a headless browser
        log_issue("Couldn't get the data of a wepage - JS needed", clean_soup)
        
    # Normal cleaner - we keep the meta tag as it creates a big loss for Wikipedia
    for a in soup.find_all('a'):  
        if not is_useful_link(a):
            a.decompose()
    tags_to_decompose = ["header", "script", "nav", "style", "popup", "footer", "button", "form", "link", "img", "video"]
    for tag in soup(tags_to_decompose):
        tag.decompose()
    for tag in soup(lambda tag: tag.has_attr("aria-hidden") and tag["aria-hidden"] == "true"):
        tag.decompose()
    
    # special cases can be listed with elif here
    remove_long = False
    if "wikipedia.org" in url:
        remove_citations(soup)
        remove_long = True
    else:
        remove_reviews(soup)
    
    text = soup.get_text()
    if remove_long: text = remove_long_sentences(text)
    text = remove_non_printable(text)
    text = remove_excess(text)

    return text

def clean_url_to_filename(url: str) -> str:
    """
    Convert URL to a suitable filename.
    """
    file_name = urlparse(url).netloc
    if file_name and "." in file_name:
        file_name = file_name.split(".")[1]
    if file_name == "":
        file_name = url[url.find(".")+1: url.find(".",url.find(".")+1)]
        if not file_name: file_name = url.replace("wwww","").replace("https://","").replace(".","")
 
    file_name = file_name.replace("/","_")
    file_name = file_name.replace("-","_")
    if file_name.startswith("_"): file_name = file_name[1:]
    return file_name

def clean_url_into_title(url: str) -> str:
    """
    Convert URL to a shortened version excluding the protocol and 'www'.

    Examples:
    "https://www.skyla.chat" will return "skyla.chat/"
    "https://chat.openai.com/sub1/sub3" will return "chat.openai.com/sub1/sub3"
    """
    parsed_url = urlparse(url)
    domain = parsed_url.netloc.replace('www.', '')
    path = parsed_url.path.strip('/')
    return f"{domain}/{path}"

def content_type_is_text(content_type) -> bool:
    """
    Used to skip if content_type.startswith("image") or content_type.startswith("application/pdf").
    Now, we skip if it's not text or xml.
    """
    if content_type.startswith("text") or content_type.startswith("application/xhtml+xml"):
        return True
    else:
        return False

def crawl_handle_fetch_result(future, data_name, memory_store) -> None:
    """
    Handles fetched data and stores it in-memory.
    """
    content = future.result()
    if content is not None:
        memory_store[data_name] = content

def crawl_website(url: str, how_many_pages = 30, memory_store = None) -> dict:
    """
    Crawl website starting from a given URL. Stores data in-memory. 
    
    Returns the dictionnary with all the content.
    """
    submitted = 0
    if not memory_store:
        memory_store = {}  # In-memory storage for crawled data
    local_domain = urlparse(url).netloc 
    queue = deque([url])
    seen = set([url])
    with concurrent.futures.ThreadPoolExecutor() as executor:
        while queue:
            url = queue.pop()
            if not check_valid_url(url):
                continue
            response = session.head(url, headers=HEADERS)
            content_type = response.headers.get("content-type", "")
            if not content_type_is_text(content_type):
                continue
            data_name = clean_url_into_title(url)
            future = executor.submit(fetch_content_url, url)
            submitted += 1
            if submitted == how_many_pages:
                break
            future.add_done_callback(lambda future: wrap_handle_fetch_result(future, data_name, memory_store))
            for link in fetch_domain_links(local_domain, url):
                if link not in seen:
                    queue.append(link)
                    seen.add(link)
    return memory_store

def fetch_hyperlinks(url: str) -> list[str]:
    """
    Fetch and returns all hyperlinks from a given URL, filtering out non-HTML content and irrelevant links.

    Returns a list with all the useful links present in a page.
    """
    try:
        response = requests.get(url, headers=HEADERS)
        if not response.headers.get('Content-Type', '').startswith('text/html'):
            return []
        html = response.text
    except Exception as e:
        log_issue(e, fetch_hyperlinks, f"Couldn't fetch the links of url {url}")
        return []

    soup = BeautifulSoup(html, 'html.parser')
    links = [link.get('href') for link in soup.find_all('a') if link.get('href')]

    filtered_links = [link for link in links if not (
        link.startswith(('javascript:', 'mailto:', '#')) or '://' not in link and not link.startswith('/')
    )]
    return filtered_links

# Fetch URL - works as a standalone
# Might want to test the driver version with selenium - driver = webdriver.Firefox()
def fetch_content_url(url: str, attempt: int = 0) -> Optional[str]:
    """
    Fetch and clean content from a webpage. 
    Returns the clean content or None if issue.
    """
    #print(f"Doing {url}")
    try:
        data = session.get(url, timeout=5)  # Using session object instead of requests
        if data.status_code == 200:
            soup = BeautifulSoup(data.text, "html.parser")
            clean = clean_soup(soup, url)
            return clean
        elif data.status_code == 429 and attempt < 2:
            # in case of too many requests (429 is rate limiting), we wait and attempt again using exponential backoff
            attempt += 1
            sleep_time = (2 ** attempt) + random.uniform(0, 1)
            time.sleep(sleep_time)
            return fetch_content_url(url, attempt)
        else:
            # "URL could not be accessed:" - @ ToDecide if we want to do smth with it
            return None
    except SSLError as e:
        log_issue(e, fetch_content_url, f"SSL/TLS error for url {url}")
        return None
    except Exception as e:
        log_issue(e, fetch_content_url, f"For url {url}")
        return None

def is_useful_link(tag) -> bool:
    """
    Mini function to check if a link is surrounded with content, hence useful, or alone.
    Return True if useful, False otherwise.
    """
   # Check previous and next siblings
    prev_sibling = tag.previous_sibling
    next_sibling = tag.next_sibling
    if prev_sibling and prev_sibling.string and prev_sibling.string.strip() != '':
        return True
    if next_sibling and next_sibling.string and next_sibling.string.strip() != '':
        return True
    # Check if link is in the middle of the parent's text (excluding the link's own text)
    parent = tag.parent
    if parent:
        text_without_link = parent.get_text().replace(tag.get_text(), '').strip()
        if text_without_link:
            return True
    return False

def fetch_domain_links(local_domain, url) -> set:
    """
    Returns the list of all unique urls of a given domain. Search start from a specific page (url).
    Doesn't return links that are not part of the domain.
    """
    clean_links = set()
    try:
        raw_links = fetch_hyperlinks(url)
        if raw_links is None:
            print("No hyperlink", fetch_domain_links, f"For {url} and {local_domain}")
            return []
        for link in set(raw_links):
            if link is None:
                continue
            valid_link = False
            # If the link is a URL, check if it is within the same domain
            if re.search(HTTP_URL_PATTERN, link):
                if urlparse(link).netloc == local_domain: # to check that the domain is the same
                    valid_link = link
            # If the link is not a URL, check if it is a relative link
            else:
                if link.startswith("/") and not link.startswith("//"):
                    link = link[1:]
                    valid_link = f"https://{local_domain}/{link}"
                elif link.startswith("#") or link.startswith("mailto:"):
                    continue
                else:
                    valid_link = urljoin(url, link)            
            if valid_link:
                if valid_link.endswith("/"):
                    valid_link = valid_link[:-1]
                clean_links.add(valid_link)
    except Exception as e:
        log_issue(e, fetch_domain_links, f"For {url} and {local_domain}")
    return clean_links


def remove_citations(soup: BeautifulSoup) -> BeautifulSoup:
    """
    Remove citation tags from a BeautifulSoup object.
    """
    for tag in soup.find_all(lambda t: t.has_attr('id') and 'cite_note' in t['id']):
        tag.decompose()
    return soup

def remove_long_sentences(text: str, max_words: int = 50) -> str:
    """
    Remove sentences from the text that have more than max_words words.

    Args:
        text (str): The input text.
        max_words (int, optional): The maximum number of words a sentence can have. Defaults to 50.

    Returns:
        str: Text with long sentences removed.
    """
    # Use regex to split text into sentences
    #re.split(r'(?<!\w\.\w.)(?<![A-Z][a-z]\.)(?<=\.|\?)\s', text)
    sentences = re.split(r'(?<!\w\.\w.)(?<![A-Z][a-z]\.)(?<=\.|\?)\s', text)
    cleaned_sentences = [sentence for sentence in sentences if len(sentence.split()) <= max_words]
    return ' '.join(cleaned_sentences)


# To be improved
def remove_reviews(soup : BeautifulSoup) -> BeautifulSoup:
    """
    Slightly risky function which removes the reviews from a Shopify store (for now - add more cases)
    """
    pattern = re.compile(r'data-verified-.*|review.*')
    for div in soup.find_all('div'):
        if div.attrs is not None: 
            div_class = div.get('class')
            if div_class is not None and any(pattern.match(class_) for class_ in div_class):
                div.decompose()
    return soup

def wrap_handle_fetch_result(future, data_name, memory_store):
    """
    Wrapper to allow adding paramaters to the callback function.
    """
    crawl_handle_fetch_result(future, data_name, memory_store)

def check_valid_url(url) -> bool:
    """
    Function which takes a string and return True if the url is valid.
    """
    try:
        result = urlparse(url)
        if len(result.netloc) <= 1: return False # Checks if the user has put a local file
        return all([result.scheme, result.netloc])
    except ValueError:
        return False

def clean_url(url) -> str:
    """
    User-submitted urls might not be perfectly fit to be processed by check_valid_url
    """
    url = url.strip()
    if not url.startswith('http'):
        url = 'https://' + url
    parsed_url = urlparse(url)

    # Clean the domain by removing any unwanted characters
    cleaned_netloc = re.sub(r'[^a-zA-Z0-9\.\-]', '', parsed_url.netloc)

    # Ensure proper percent-encoding of the path component
    unquoted_path = unquote(parsed_url.path)
    quoted_path = quote(unquoted_path)

    cleaned_url = urlunparse(parsed_url._replace(netloc=cleaned_netloc, path=quoted_path))
    return cleaned_url

def get_local_domain(from_url) -> str:
    """
    Get the local domain from a given URL.
    Will return the same domain for https://chat.openai.com/chat" and https://openai.com/chat".
    """
    try:
        netloc = urlparse(from_url).netloc
        parts = netloc.split(".")
        if len(parts) > 2:
            domain = parts[-2]
        else:
            domain = parts[0]
        print("URL: ", from_url, " Domain: ", str(domain))
        return str(domain)
    except Exception as e:
        log_issue(e, get_local_domain, f"For {from_url}")

# *************************************************************
if __name__ == "__main__":
    pass


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="scrapwithgpt",
    version="0.1.2", # need to increment this everytime otherwise Pypi will not accept the new version
     url='https://github.com/HenryObj/scrapwithgpt',
    packages=find_packages(),
    install_requires=[
        "openai>=1.9.0",
        "tiktoken>=0.5.2",
        "requests>=2.31.0",
        "bs4",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
)